/*#################################################################################################
	Fr		:	
	Version	:	1.0
	Datum	:	
	autor	:	
	Hardware:	ATXmega128A3-U, 32MHz bei 16Mhz extern
	Aufgabe	:   
//################################################################################################*/


#include <avr/io.h>
#include <avr/wdt.h>
#include <avr/power.h>
#include <stdlib.h>
#include <stdio.h>
#include <stddef.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>

#include "macros.h"
#include "avr_compiler.h"
#include "clksys_driver.h"

#include "Descriptors.h"
#include "usb.h"
#include "usb_pipe.h"

USB_PIPE(ep_in,  0x81 | USB_EP_PP, USB_EP_TYPE_BULK_gc, 64, 8, PIPE_ENABLE_FLUSH);
USB_PIPE(ep_out, 0x02 | USB_EP_PP, USB_EP_TYPE_BULK_gc, 64, 8, 0);


//#################################### Variablen	
volatile	int 		timer_blink	= 100,
						flag_blink	= 0;				// timer fr echtzeit ausfhrung
uint8_t counter = 0;
												
//########################################################################################## init_clock
void init_clock( void )
{
	CLKSYS_XOSC_Config( OSC_FRQRANGE_12TO16_gc, false, OSC_XOSCSEL_XTAL_16KCLK_gc );// Quarz config
	CLKSYS_Enable( OSC_XOSCEN_bm );													// enable oscillator
	CLKSYS_PLL_Config( OSC_PLLSRC_XOSC_gc, 8 );										// config PLL - Quarz + 8x -->128Mhz
	CLKSYS_Enable( OSC_PLLEN_bm );													// PLL enable
	CLKSYS_Prescalers_Config( CLK_PSADIV_1_gc, CLK_PSBCDIV_2_2_gc );				// Prescaller A, B, C --> Mainclock 32Mhz
	while ( !(CLKSYS_IsReady( OSC_PLLRDY_bm )) ) ;									// Wait for it to stabilize

	CLKSYS_Main_ClockSource_Select( CLK_SCLKSEL_PLL_gc );							// Source for Main Clock
	CLKSYS_Disable( OSC_XOSCEN_bm );												// Oscillator disable
} 
//########################################################################################## initialisierung
void initialisierung(void)
{	
	cli();
 	//init_clock();	
	 		
	USB_ConfigureClock();
			 	
  	//####################################### Ports 	
  	sbi(PORTE.DIR,2);	sbi(PORTE.OUT,2);	// Ausgang	LED 1
  	sbi(PORTE.DIR,3);	sbi(PORTE.OUT,3);	// Ausgang	LED	2
  	sbi(PORTE.DIR,4);	sbi(PORTE.OUT,4);	// Ausgang	LED 3
  	sbi(PORTE.DIR,5);	sbi(PORTE.OUT,5);	// Ausgang	LED	4	
  	sbi(PORTE.DIR,6);	sbi(PORTE.OUT,6);	// Ausgang	LED	5
		
	//####################################### TimerF0 - 1000 hz
	TCF0.CNT		= 0x00;
	TCF0.CTRLA		= TC_CLKSEL_DIV1024_gc;				// Vorteiler	1024
	TCF0.CTRLB		= TC_WGMODE_NORMAL_gc;				// Modus		Normal
	TCF0.PER		= 32;								// Top-Wert		1000 Hz	
	TCF0.INTCTRLA	= 0b00000011;						// Interrupt konfigurieren



	//####################################### USB
	// Enable USB interrupts
	USB.INTCTRLA = /*USB_SOFIE_bm |*/ USB_BUSEVIE_bm | USB_INTLVL_MED_gc;
	USB.INTCTRLB = USB_TRNIE_bm | USB_SETUPIE_bm;

	USB_Init();

	//####################################### enable Interrupts 
	PMIC.CTRL |= PMIC_HILVLEN_bm |PMIC_MEDLVLEN_bm|PMIC_LOLVLEN_bm;		//Interrupts (Highlevel,Mediumlevel und Lowlevel freigeben)	
	sei();
}
//##########################################################################################
void EVENT_USB_Device_ConfigurationChanged(uint8_t config)
{
	usb_pipe_init(&ep_in);
	usb_pipe_init(&ep_out);

	TCC0.CTRLA = TC_CLKSEL_DIV1024_gc; // 31.25KHz = 0.032ms
	TCC0.INTCTRLA = TC_OVFINTLVL_LO_gc; // interrupt on timer overflow
	TCC0.PER = 1563; // ~50ms
	TCC0.CNT = 0;
}
//##########################################################################################
ISR(USB_BUSEVENT_vect)
{
	if (USB.INTFLAGSACLR & USB_SOFIF_bm)
	{
		USB.INTFLAGSACLR = USB_SOFIF_bm;
	}
	else if (USB.INTFLAGSACLR & (USB_CRCIF_bm | USB_UNFIF_bm | USB_OVFIF_bm))
	{
		USB.INTFLAGSACLR = (USB_CRCIF_bm | USB_UNFIF_bm | USB_OVFIF_bm);
	}
	else if (USB.INTFLAGSACLR & USB_STALLIF_bm)
	{
		USB.INTFLAGSACLR = USB_STALLIF_bm;
	}
	else
	{
		USB.INTFLAGSACLR = USB_SUSPENDIF_bm | USB_RESUMEIF_bm | USB_RSTIF_bm;
		USB_Evt_Task();
	}
}
//##########################################################################################
ISR(USB_TRNCOMPL_vect)
{
	USB.FIFOWP = 0;
	USB.INTFLAGSBCLR = USB_SETUPIF_bm | USB_TRNIF_bm;
	USB_Task();
	usb_pipe_handle(&ep_in);
	usb_pipe_handle(&ep_out);
}
//##########################################################################################
ISR(TCC0_OVF_vect)
{
	if (usb_pipe_can_read(&ep_out) && usb_pipe_can_write(&ep_in))
	{
		PORTR.OUTTGL = 2;
	}
	while (usb_pipe_can_read(&ep_out) && usb_pipe_can_write(&ep_in))
	{
		uint8_t v = usb_pipe_read_byte(&ep_out);
		if (v == 0)
		{
			usb_pipe_flush(&ep_in);
		}
		else
		{
			usb_pipe_write_byte(&ep_in, v);
		}
	}
}
//########################################################################################## Event handler for the library USB Control Request reception event.
bool EVENT_USB_Device_ControlRequest(USB_Request_Header_t* req)
{
	if ((req->bmRequestType & CONTROL_REQTYPE_TYPE) == REQTYPE_VENDOR)
	{
		if (req->bRequest == 0xBB)
		{
			USB_enter_bootloader();
		}
	}
	return false;
}
//########################################################################################## Int Timer 1000 hz		
ISR(TCF0_OVF_vect)															
{	
	timer_blink--;		if (timer_blink		<0)	{timer_blink = 1000;	flag_blink=1;}		// timer fr Echtzeitablauf				
}		
//##########################################################################################
int main (void) 
{   		
	initialisierung();						// Load controller setup
	
	while(1) 
	{           
	//####################
	if (flag_blink)	
	{	
		flag_blink	= 0;	
 		toggle(PORTE.OUT,4);																				// LED blinken
	}		
	

	//#################### while end  
	}                                   
}




